import pygame
import numpy as np
import pandas as pd


class Grapher:
    def __init__(self, handler, size):
        self.handler = handler
        self.graphs = []
        self.size = size

        self.initialize_graphs()

    def initialize_graphs(self):
        for i, graph in enumerate(self.graphs):
            # do a better job assigning the positions of multiple graphs
            self.graphs[i].pos = (200, 100)
            self.graphs[i].size = (500, 500)

    def render(self):
        for graph in self.graphs:
            graph.render()



import pygame
import math

# import palletts as p

import cofffee.palletts as p


class text:
    def __init__(self, color=-1, font="/Users/brianmasse/Library/Fonts/Monoid-Retina.ttf", fontSize=10, text=-1):
        self.color = color
        self.font = font
        self.fontSize = fontSize
        self.text = text

        self.parent = None
        self.rendering = True

        self.default = False

    def __reinit__(self, parent):
        self.parent = parent
        if self.color == -1:
            self.default = True
            self.color = parent.pallett.text_RGB

    def __color__(self, color):
        if color == - 1 or not self.default:
            return self.color
        else:
            return color

    def __text__(self, text):
        if self.text == -1:
            return text
        else:
            return self.text

    def render(self, message, pos, color=-1, alignmentX="center", alignmentY="center"):
        if self.rendering:
            font = pygame.font.Font(self.font, self.fontSize)
            text = font.render(self.__text__(message),
                               True, self.__color__(color))
            textRect = text.get_rect()

            y = self.parent.graph.handler.height - pos[1]
            textRect.center = (pos[0], y)
            if alignmentX == "left":
                textRect.left = pos[0]
            elif alignmentX == "right":
                textRect.right = pos[0]
            if alignmentY == "top":
                textRect.top = y
            elif alignmentY == "bottom":
                textRect.bottom = y

            self.parent.graph.handler.surface.blit(text, textRect)

    # USER FUNCTIONS
    def update_visibility(self, vis):
        self.rendering = vis
        return self.parent


class line:
    def __init__(self, color=-1, stroke=1):
        self.color = color
        self.stroke = stroke

        self.parent = None
        self.rendering = True

        self.default = False

    def __reinit__(self, parent):
        self.parent = parent
        if self.color == -1:
            self.default = True
            self.color = parent.pallett.text_RGB

    def __color__(self, color):
        if color == - 1 or not self.default:
            return self.color
        else:
            return color

    def render(self, p, p2, color=-1):
        if self.rendering:
            pos = (p[0], self.parent.graph.handler.height - p[1])
            pos2 = (p2[0], self.parent.graph.handler.height - p2[1])

            pygame.draw.line(self.parent.graph.handler.screen,
                             self.__color__(color), pos, pos2, self.stroke)

    # USER FUNCTIONS
    def update_visibility(self, vis):
        self.rendering = vis
        return self.parent


class point:
    def __init__(self, color=-1, radius=2, stroke=0, stroke_color=-1, shape=-1):
        self.color = color

        self.radius = radius
        self.stroke = stroke
        self.stroke_color = stroke_color
        self.shape = shape

        self.handler = None
        self.parent = None

        self.rendering = True
        self.default = False

    def __reinit__(self, parent):
        self.parent = parent
        self.handler = parent.graph.handler
        if self.stroke_color == -1:
            self.stroke_color = parent.pallett.text_RGB
        if self.color == -1:
            self.default = True
            self.color = parent.pallett.prim_RGB

    def __color__(self, color):
        if color == - 1 or not self.default:
            return self.color
        else:
            return color

    def __render_polygon__(self, center, color):
        coords = []
        outlines = []
        for side in range(0, self.shape):
            theta = 2 * math.pi * (side / self.shape)
            point = ((math.cos(theta) * self.radius),
                     (math.sin(theta) * self.radius))
            outline = (math.cos(theta) * (self.radius + self.stroke),
                       math.sin(theta) * (self.radius + self.stroke))
            coords.append((point[0]+center[0], point[1]+center[1]))
            outlines.append((outline[0]+center[0], outline[1]+center[1]))

        pygame.draw.polygon(self.handler.surface, self.stroke_color, outlines)
        pygame.draw.polygon(self.handler.surface, color, coords)

    def render(self, p, color=-1):
        if self.rendering:
            pos = (p[0], self.handler.height - p[1])
            if self.shape == -1:
                pygame.draw.circle(
                    self.handler.surface, self.stroke_color, pos, self.radius + self.stroke)
                pygame.draw.circle(self.handler.surface,
                                   self.__color__(color), pos, self.radius)
            else:
                self.__render_polygon__(pos, self.__color__(color))

    # USER FUNCTIONS

    def update_visibility(self, vis):
        self.rendering = vis
        return self.parent


class domain:
    def __init__(self, pos, size, padding=(75, 30), parent=None, pallett=p.green_tea):
        self.pos = pos
        self.size = size
        self.padding = padding

        self.pallett = pallett
        self.parent = parent

        self.rendering = True

    def __reinit__(self, parent):
        self.parent = parent
        self.pallett = parent.pallett

    def render(self):
        if self.rendering:

            rect = pygame.Rect(
                self.pos[0] - self.padding[0],
                self.pos[1] + self.size[1] + self.padding[1],
                self.size[0] + (self.padding[0] * 2),
                self.size[1] + (self.padding[1] * 2)
            )

            self.parent.graph.handler.render_rect(rect, self.pallett.back_RGB)

    # User Functions

    def update_padding(self, padding):
        self.padding = padding
        return self.parent

    def update_visibility(self, vis):
        self.rendering = vis
        return self.parent


default_domain = domain(
    pos=(100, 100),
    size=(500, 500),
)



# from optparse import Values
# import pygame
# import pandas as pd
# import numpy as np
# import sys

# import color as c
# import palletts as p

# import pyg as pg
# import graphers as g
# import grapher as go
# import graph_props as m
# import axis as ax

# xls = pd.ExcelFile(
#     "cofffee/data/coffee.xlsx"
# )
# data = pd.read_excel(xls, "Coffee Chain")

# width = 1700
# height = 900

# active_pallett = p.planetary_punch

# handler = pg.handler(width, height, title="Coffee Shop Data")
# main = go.Grapher(handler, (1700, 1000))


# line = m.line(
#     stroke=5)
# labels = labels = m.text(
#     font="/Users/brianmasse/Library/Fonts/Lulo Clean One Bold.otf",
#     fontSize=10)
# axis = ax.axis(
#     line=line,
#     ticks=line,
#     labels=labels)

# marketing = g.area(main, data, "count()", "Marketing", "Product Type",
#                    domain=m.domain(
#                        pos=(100, 50),
#                        size=(width / 4, height - 100)),
#                    line=line,
#                    title=m.text(
#                        font="/Users/brianmasse/Library/Fonts/Lulo Clean One Bold.otf",
#                        fontSize=10,
#                        text="Quantity of Marketing Campaigns for Various Beverages"),
#                    value_axis=axis,
#                    series_axis=axis,
#                    pallett=active_pallett).point.update_visibility(False)


# sales = g.area(main, data, "count()", "Sales", "Product Type",
#                domain=m.domain(
#                    pos=(200 + width / 4, 50),
#                    size=(width / 4, height - 100)),
#                point=m.point(
#                    stroke=2,
#                    radius=7),
#                title=m.text(
#                    fontSize=15,
#                    text="Sales of Various Beverages ($)"),
#                pallett=active_pallett)


# axis1 = ax.axis(
#     line=m.line(stroke=0))
# axis2 = ax.axis(
#     ticks=m.line(stroke=0))

# expenses = g.area(main, data, "count()", "Total Expenses", "Product Type",
#                   domain=m.domain(
#                       pos=(350 + width / 2, 50),
#                       size=(width / 4, height - 100)),
#                   title=m.text(
#                       fontSize=20,
#                       text="Expenses for Various Beverages ($)"),
#                   value_axis=axis1,
#                   series_axis=axis2,
#                   pallett=active_pallett).point.update_visibility(False)


# main.graphs = [marketing, sales, expenses]
# main.handler.screen.fill(active_pallett.back_RGB)
# main.render()


# handler.start()



# import color as c

import cofffee.color as c

# green tea

prim = c.color(190, 232, 194)
second = c.color(209, 255, 252)

grad = prim.return_color_grad(second, 100)

back = c.color(140, 171, 160)
text = c.color(73, 89, 84)

green_tea = c.pallet(grad, prim, second, back, back, text)

# cofffee Bean

prim = c.color(179, 69, 18, 0.5)
second = c.color(89, 42, 20)

grad = prim.return_color_grad(second, 100)

back = c.color(230, 171, 145)
text = c.color(61, 22, 4)

cofffee_bean = c.pallet(grad, prim, second, back, back, text)

# Blue Lagoon

prim = c.color(38, 201, 147)
second = c.color(45, 212, 227)

grad = prim.return_color_grad(second, 100)

back = c.color(187, 240, 225)
text = c.color(43, 92, 87)

blue_lagoon = c.pallet(grad, prim, second, back, back, text)


# Milk Shake

prim = c.color(252, 48, 79)
second = c.color(59, 176, 255)

grad = prim.return_color_grad(second, 100)

back = c.color(245, 218, 228)
text = c.color(190, 69, 121)

milk_shake = c.pallet(grad, prim, second, back, back, text)

# Planetary Punch

prim = c.color(214, 161, 255)
second = c.color(129, 240, 179)

grad = prim.return_color_grad(second, 100)

back = c.color(20, 20, 20)
text = c.color(255, 255, 255)

planetary_punch = c.pallet(grad, prim, second, back, back, text)



import math


class color:

    hex_values = ["0", "1", "2", "3", "4", "5", "6",
                  "7", "8", "9", "A", "B", "C", "D", "E", "F"]

    def __init__(self, R, G, B, A=1):
        self.R = R
        self.G = G
        self.B = B
        self.A = A

    def return_color_between(self, color2, perc):
        r_change = color2.R - self.R
        g_change = color2.G - self.G
        b_change = color2.B - self.B
        a_change = color2.A - self.A

        r = self.R + (r_change * perc)
        g = self.G + (g_change * perc)
        b = self.B + (b_change * perc)
        a = self.A + (a_change * perc)

        return color(r, g, b, a)

    def __convert_tuple__(self, tuple):
        string = '('
        for i, item in enumerate(tuple):
            if i == len(tuple) - 1:
                string = string + str(item)
            else:
                string = string + str(item) + ","
        return string + ")"

    def return_string_in(self, code):
        if code == "RGB":
            return "rgba" + self.__convert_tuple__((self.R, self.G, self.B, self.A * 255))
        if code == "HSB":
            return "hsb" + self.__convert_tuple__((self.R, self.G, self.B, self.A))
        if code == "HEX":
            hex1 = self.__return_hex(self.R)
            hex2 = self.__return_hex(self.G)
            hex3 = self.__return_hex(self.B)
            hex4 = self.__return_hex(self.A)
            return "#{}{}{}{}{}{}".format(hex1[0], hex1[1], hex2[0], hex2[1], hex3[0], hex3[1], hex4[0], hex4[1])

    def return_color_in(self, code):
        if code == "RGB":
            return (self.R, self.G, self.B, self.A * 255)
        if code == "HSB":
            return (self.R, self.G, self.B, self.A)
        if code == "HEX":
            hex1 = self.__return_hex(self.R)
            hex2 = self.__return_hex(self.G)
            hex3 = self.__return_hex(self.B)
            hex4 = self.__return_hex(self.A)
            return "#{}{}{}{}{}{}".format(hex1[0], hex1[1], hex2[0], hex2[1], hex3[0], hex3[1], hex4[0], hex4[1])

    def __return_hex(self, component):
        rounded = math.floor(component / 16)
        remainder = (component / 16) - rounded

        hex1 = self.hex_values[rounded]
        hex2 = self.hex_values[math.floor(remainder * 16)]
        return (hex1, hex2)

    def __return_primes_for_RGB(self, hue, C, X):
        if 0 <= hue and hue < 60:
            return (C, X, 0)
        elif 60 <= hue and hue < 120:
            return (X, C, 0)
        elif 120 <= hue and hue < 180:
            return (0, C, X)
        elif 180 <= hue and hue < 240:
            return (0, X, C)
        elif 240 <= hue and hue < 300:
            return (X, 0, C)
        elif 300 <= hue and hue < 360:
            return (C, 0, X)

    def return_RGB(self):
        value = self.B / 100
        sat = self.G / 100

        C = sat * value
        X = C * (1 - abs(((self.R / 60) % 2) - 1))
        m = value - C

        primes = self.__return_primes_for_RGB(self.R, C, X)
        r = (primes[0] + m) * 255
        g = (primes[1] + m) * 255
        b = (primes[2] + m) * 255
        return (r, g, b)

    def return_HSB(self):
        r = self.R / 255
        g = self.G / 255
        b = self.B / 255

        cmax = max(r, g, b)
        cmin = min(r, g, b)
        delta = cmax - cmin

        h = 0
        s = 0
        v = 0

        # hue:
        if delta == 0:
            h = 0
        elif cmax == r:
            h = 60 * (((g - b) / delta) % 6)
        elif cmax == g:
            h = 60 * (((b - r) / delta) + 2)
        elif cmax == b:
            h = 60 * (((r - g) / delta) + 4)

        # saturation
        if cmax == 0:
            s = 0
        else:
            s = delta / cmax

        # value
        v = cmax

        return (h, s * 100, v * 100)

    def return_color_grad(self, second_color, steps):
        colors = []
        for step in range(0, steps):
            interval = step / (steps - 1)
            color = self.return_color_between(second_color, interval)
            colors.append(color.return_color_in("HEX"))
        return colors


class pallet:
    def __init__(self, grad, primary_color, secondary_color, background, secondary_background, text):
        self.grad = grad
        self.primary_color = primary_color
        self.secondary_color = secondary_color
        self.background = background
        self.secondary_background = secondary_background
        self.text = text

        self.prim_RGB = primary_color.return_color_in("RGB")
        self.second_RGB = secondary_color.return_color_in("RGB")
        self.back_RGB = background.return_color_in("RGB")
        self.second_back_RGB = secondary_background.return_color_in("RGB")
        self.text_RGB = text.return_color_in("RGB")

        self.prim_HEX = primary_color.return_color_in("HEX")
        self.second_HEX = secondary_color.return_color_in("HEX")
        self.back_HEX = background.return_color_in("HEX")
        self.second_back_HEX = secondary_background.return_color_in("HEX")
        self.text_HEX = text.return_color_in("HEX")



import math

# import graph_props as m

import cofffee.graph_props as m


def round_dig(num, dig):
    num = float(num)
    num = num * (math.pow(10, dig))
    num = round(num)
    return num / (math.pow(10, dig))


class axis():
    def __init__(self, line=-1, ticks=-1, labels=-1):
        self.parent = None

        self.line = self.__return_default__(line, m.line())
        self.ticks = self.__return_default__(ticks, m.line())
        self.labels = self.__return_default__(labels, m.text())

    def __reinit__(self, parent):
        self.parent = parent
        self.line.__reinit__(self.parent)
        self.ticks.__reinit__(self.parent)
        self.labels.__reinit__(self.parent)

    def __return_default__(self, obj, default):
        if obj == -1:
            return default
        return obj


class Axis:
    def __init__(self, parent, series, dir, interval=-1, min=0, axis=-1):
        self.parent = parent
        self.interval = interval
        self.series = series

        self.dir = dir
        self.min = min

        self.axis = self.__return_default_axis__(axis)
        self.axis.__reinit__(self.parent)

        self.rendering = True

        self.__series_type__ = self.__return_series_type__()

        # the interval is the steps that a value must take to reach its end by the end of some space
        if interval == -1 and (self.__series_type__ == "int" or self.__series_type__ == "float"):
            self.interval = self.__return_auto_interval__()

    def __return_default_axis__(self, ax):
        if ax == -1:
            return axis()
        return ax

    def __return_series_type__(self):
        string = str(self.series[0])
        string = string.lstrip("-")
        if string.isdigit():
            return "float"

    def __return_auto_interval__(self):
        return self.parent.size[self.parent.dir] / (self.series[-1] - self.series[0])

    # for aligning axis labels
    def __return_alignment__(self):
        if self.dir == 0:
            return "center"
        else:
            return "right"

    def render(self):
        if self.rendering:
            # domain

            origin = (self.parent.pos[0] - 10, self.parent.pos[1] - 10)
            x = math.cos(self.dir * (math.pi / 2))
            y = math.sin(self.dir * (math.pi / 2))
            end = (origin[0] + (self.parent.size[0] + 10) * x,
                   origin[1] + (self.parent.size[1] + 10) * y)
            self.axis.line.render(origin, end, self.parent.pallett.text_RGB)

            # labels

            if self.__series_type__ == "int" or self.__series_type__ == "float":

                spacing = (
                    (self.series[-1] - self.series[0]) / self.parent.size[self.dir]) * 50
                pos = int(self.series[0])
                # render all numbers
                while pos <= self.series[-1]:
                    prim = origin[self.dir] + \
                        ((pos - self.min) * self.interval)
                    second = origin[abs(self.dir - 1)] - 10

                    label_pos = ((prim * x) + (second * y),
                                 (prim * y) + (second * x))

                    self.__render_ticks__(label_pos, x, y)
                    self.axis.labels.render(str(round_dig(
                        pos, 1)), label_pos, self.parent.pallett.text_RGB, self.__return_alignment__())
                    pos += spacing

            # render all series
            else:
                spacing = self.parent.size[self.dir] / len(self.series)
                for i, label in enumerate(self.series):
                    label_pos = (origin[0] + ((i * spacing) * x) - (10 * y),
                                 origin[1] + ((i * spacing) * y) - (10 * x))
                    self.__render_ticks__(label_pos, x, y)
                    self.axis.labels.render(
                        str(label), label_pos, self.parent.pallett.text_RGB, self.__return_alignment__())

    def __render_ticks__(self, label_pos, x, y):
        tick_pos = (label_pos[0] + (10 * y), label_pos[1] + (10 * x))
        tick_pos2 = (label_pos[0] + (5 * y), label_pos[1] + (5 * x))
        # tick_pos2 = ( label_pos[0] -   , label_pos[1] - 10 )

        self.axis.ticks.render(tick_pos, tick_pos2)

    # USER FUNCTIONS

    def update_visibility(self, vis):
        self.rendering = vis
        return self.parent



import pygame


class handler:
    def __init__(self, width=0, height=0, title=""):
        pygame.init()

        self.width = width
        self.height = height

        self.title = title

        Icon = pygame.image.load(
            "cofffee/Extra/icon.png")

        self.running = True
        self.screen = pygame.display.set_mode((self.width, self.height))
        pygame.display.set_caption(self.title)
        pygame.display.set_icon(Icon)

        self.surface = self.__return_surface__()

    def start(self):
        while self.running:
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    self.running = False
            pygame.display.flip()
        pygame.quit()

    # OBJECT RENDERING

    def __return_surface__(self):
        return pygame.Surface((self.width, self.height), pygame.SRCALPHA)

    def render_que(self):
        self.screen.blit(self.surface, (0, 0))
        self.surface = self.__return_surface__()

    def render_text(self, message, position, color, size, alignment="center"):
        font = pygame.font.Font(
            "/Users/brianmasse/Library/Fonts/Monoid-Retina.ttf", size)
        text = font.render(message, True, color)
        textRect = text.get_rect()

        textRect.center = (position[0], self.height - position[1])
        if alignment == "left":
            textRect.left = position[0]
        elif alignment == "right":
            textRect.right = position[0]

        self.screen.blit(text, textRect)

    def render_rect(self, rect, color):
        new_rect = pygame.Rect(rect.left, self.height -
                               rect.top, rect.width, rect.height)
        pygame.draw.rect(self.screen, color, new_rect)

    def render_line(self, start_pos, end_pos, color, thickness=2):
        pygame.draw.line(self.screen, color, (
            start_pos[0], self.height - start_pos[1]), (end_pos[0], self.height - end_pos[1]), thickness)



import sys
import pygame
from operator import sub
from matplotlib.pyplot import text
import pandas as pd
import random
import numpy as np
import math

# import color as c
# import axis as ax
# import palletts as p
# import graph_props as m

import cofffee.color as c
import cofffee.axis as ax
import cofffee.palletts as p
import cofffee.graph_props as m


def round_dig(num, base=5, dig=1):
    num = float(num)
    num = num / (math.pow(base, dig))
    num = round(num)
    return num * (math.pow(base, dig))


class distribution:
    def __init__(self, graph, data, x, series, dir=1, domain=m.default_domain, point=-1, pallett=p.green_tea, title="", ):
        self.graph = graph
        self.data = data
        self.x = data[x]
        self.dir = dir

        self.__ordered__ = self.__sort_df__(self.x)
        self.__min__ = self.__find_extremum__(min, 0)
        self.__max__ = self.__find_extremum__(max, -1)

        self.series = data[series]
        self.__unique_series__ = self.unique_series(self.series)
        self.series_data = self.create_series()

        self.pos = domain.pos
        self.size = domain.size

        self.value_axis = ax.Axis(
            self, [self.__min__, self.__max__], self.dir, min=self.__min__)
        self.series_axis = ax.Axis(
            self, self.__return_series_names__(), abs(self.dir - 1),  min=self.__min__)

        self.pallett = pallett

        self.domain = domain
        self.domain.__reinit__(self)
        self.point = self.__return_default_point__(point)
        self.point.__reinit__(self)

        self.title = title

    def __return_default_point__(self, point):
        if point == -1:
            return m.point(self.pallett.prim_RGB)
        return point

    def __find_extremum__(self, func, index):
        extremum = 0
        for series in self.__ordered__:
            extremum = func(series[index], extremum)
        return extremum

    def __sort_df__(self, df):
        returning = []
        rotated = np.rot90(df.to_numpy())
        for series in rotated:
            returning.append(
                list(np.sort(series[np.logical_not(np.isnan(series))])))
        return returning

    def __return_series_names__(self):
        returning = []
        for series in self.series_data:
            returning.append(series.name)
        return returning

    def unique_series(self, series):
        a = series.to_numpy()
        unique_series = np.unique(a)
        return unique_series

    def create_series(self):
        returning = []
        index = 0

        for series in self.__unique_series__:
            for column in self.x:
                rows = self.series == series
                selection = self.data.loc[rows, column]
                # series_obj = self.Series(self, "" + series + ", " + column, selection, index)
                series_obj = self.Series(self, "" + series, selection, index)
                returning.append(series_obj)
                index += 1

        return returning

    # RENDER

    def render(self):
        self.domain.render()
        self.graph.handler.render_text(self.title, (self.pos[0] + (
            self.size[0] / 2),  self.pos[1] + (self.size[1])), self.pallett.text_RGB, 20)

        for series in self.series_data:
            series.render()

        self.value_axis.render()
        self.series_axis.render()

        self.graph.handler.render_que()

    class Series:
        def __init__(self, parent, name, x, index):
            self.parent = parent
            self.name = name
            self.x = np.sort(x.to_numpy())
            self.index = index

            self.random = -1

        def render(self):

            series_space = (self.parent.size[abs(
                self.parent.dir - 1)] / len(self.parent.series_data))

            steps = 1 / len(self.parent.__unique_series__)

            for value in self.x:

                rand = series_space / 3
                if self.random != -1:
                    rand = self.random
                series = (self.index * series_space) + random.uniform(0, rand)

                # print(self.parent.value_axis.interval)
                value = ((value - self.parent.__min__) *
                         self.parent.value_axis.interval) + self.parent.pos[self.parent.dir]
                series += self.parent.pos[abs(self.parent.dir - 1)]

                color = self.parent.pallett.primary_color.return_color_between(
                    self.parent.pallett.secondary_color, steps * self.index).return_color_in("RGB")

                if not np.isnan(value):

                    if self.parent.dir == 0:
                        self.parent.point.render((value, series), color)
                    else:
                        self.parent.point.render((series, value), color)

    # user functions:

    def update_series_rand(self, rand):
        for series in self.series_data:
            series.random = rand
        return self


class area:
    def __init__(self, graph, data, x, y, series, dir=0, domain=m.default_domain, point=-1, line=-1, title=-1, value_axis=-1, series_axis=-1, pallett=p.green_tea,):
        self.graph = graph
        self.data = data
        self.dir = dir

        self.series = data[series]
        self.__unique_series__ = self.unique_series(data[series])

        # this does not gaurentee the data is ready
        self.x = self.__order_data__(x)
        self.y = self.__order_data__(y)

        # get the count of data, if it was specified (this will also override the dir, if one series is the count)
        # By this point the dir is known
        self.x = self.__count__(self.y, y, self.x, 1)
        self.y = self.__count__(self.x, x, self.y, 0)
        self.opp = abs(self.dir - 1)

        # this needs the prepared data
        self.series_data = self.create_series()

        self.__x_min__ = self.__find_extremum__(min, self.x)
        self.__y_min__ = self.__find_extremum__(min, self.y)
        self.__x_max__ = self.__find_extremum__(max, self.x)
        self.__y_max__ = self.__find_extremum__(max, self.y)

        # for the series render function
        self.mins = [self.__x_min__, self.__y_min__]
        self.maxs = [self.__x_max__, self.__y_max__]

        self.pos = domain.pos
        self.size = domain.size

        self.pallett = pallett

        self.value_axis = ax.Axis(self, self.__return_values__(
        ), self.dir, min=self.__x_min__, axis=value_axis)
        self.series_axis = ax.Axis(self, self.__return_series_names__(
        ), self.opp, min=self.__y_min__, axis=series_axis)

        self.domain = domain
        self.domain.__reinit__(self)
        self.point = self.__return_default__(
            point, m.point(self.pallett.prim_RGB))
        self.point.__reinit__(self)
        self.line = self.__return_default__(
            line, m.line(self.pallett.text_RGB))
        self.line.__reinit__(self)
        self.title = self.__return_default__(title, m.text(text=""))
        self.title.__reinit__(self)

    def __return_default__(self, obj, default):
        if obj == -1:
            return default
        return obj

    # find the value series depending on the dir
    def __return_values__(self):
        if self.dir == 0:
            return self.x
        else:
            return self.y

    # orders the data sets, unless they are counts, that will be dealt with later
    def __order_data__(self, series):
        data = ["count()"]
        if series != "count()":
            data = self.data[series]

        if self.__is_numeric__(data):
            return self.__sort_df__(data)
        return data

    # Get the count if it was passed in:
    def __count__(self, counted_series, cs_name, counting_series, dir):
        returning = counting_series
        if str(counting_series[0]) == "count()":
            returning = []

            # The value should always be the primary axis (dir = 0)
            self.dir = dir
            for series in self.__unique_series__:
                counts = []
                rows = self.series == series
                sub_section = self.data.loc[rows, cs_name]
                sub_section = pd.Series(map(round_dig, sub_section))

                for value in counted_series:
                    count = (sub_section == round_dig(value)).sum()
                    counts.append(count)
                returning.append(counts)
        return returning

    # DF management
    def __is_numeric__(self, arr):
        string = str(arr[0])
        if "-" in string:
            string = string.lstrip("-")
        return string.isdigit()

    def __sort_df__(self, df):
        returning = []
        arr = df.to_numpy()
        arr = arr[np.logical_not(np.isnan(arr))]
        returning = np.sort(arr)
        return returning

    def unique_series(self, series):
        a = series.to_numpy()
        unique_series = np.unique(a)
        return unique_series

    def __find_extremum__(self, func, arr):
        extremum = 0
        if isinstance(arr[0], list):
            for series in arr:
                extremum = func(extremum, self.__find_extremum__(func, series))
        else:
            if self.__is_numeric__(arr):
                for value in arr:
                    extremum = func(value, extremum)
        return extremum

    # SERIES

    def __return_series_names__(self):
        returning = []
        for series in self.series_data:
            returning.append(series.name)
        return returning

    def create_series(self):
        returning = []
        index = 0

        for series in self.__unique_series__:
            # depending on the dir, the value series gets the y or x series, and same with the series series

            series_obj = self.Series(self, "", self.x, self.y[index], index)
            if self.dir == 1:
                series_obj = self.Series(
                    self, str(series), self.y, self.x[index], index)
            returning.append(series_obj)
            index += 1

        return returning

    # RENDER

    def render(self):
        self.domain.render()
        self.title.render("", (self.pos[0] + (
            self.size[0] / 2),  self.pos[1] + (self.size[1] + 20)), self.pallett.text_RGB, alignmentY="bottom")

        for series in self.series_data:
            series.render()

        self.value_axis.render()
        self.series_axis.render()

        self.graph.handler.render_que()

    class Series:
        def __init__(self, parent, name, x, y, index):
            self.parent = parent
            self.name = name
            self.x = x
            self.y = y
            self.index = index

        def render(self):

            points = []
            dir = self.parent.dir
            opp = self.parent.opp

            # Basically:
            # There are two coordinate points the value (the dir axis) and the count (the opp axis), both can be on either the x or y axis, and so I need to supply the right x or y data depdning on the orientation of the graph ( this is what all the [dir]  and [opp] are)
            # The self.x and self.y are already changed depending on what the direction is, as they are passed in considering the directin, so I dont need to do any work here

            value_space = (
                self.parent.size[dir] / (self.parent.maxs[dir] - self.parent.mins[dir]))

            count_space = (self.parent.size[opp]) / (len(self.parent.__unique_series__) * (
                20 + self.parent.maxs[opp] - self.parent.mins[opp]))
            total_count_space = (
                self.parent.size[opp]) / (len(self.parent.__unique_series__))

            steps = 1 / len(self.parent.__unique_series__)
            color = self.parent.pallett.primary_color.return_color_between(
                self.parent.pallett.secondary_color, steps * self.index).return_color_in("RGB")

            for i in range(0, len(self.x)):

                value = ((self.x[i] - self.parent.mins[dir])
                         * value_space) + self.parent.pos[dir]
                count = ((self.y[i] - self.parent.mins[opp]) * count_space)

                count += (total_count_space * self.index) + \
                    self.parent.pos[opp]

                point = 0
                if dir == 1:
                    point = (count, value)
                if dir == 0:
                    point = (value, count)
                points.append(point)
                self.parent.point.render(point, color)

            points.sort(key=lambda y: y[dir])
            for i in range(0, len(points) - 1):

                self.parent.line.render(points[i], points[i + 1], color)


# import cofffee.color as c
# import cofffee.palletts as p

# import cofffee.pyg as pg
# import cofffee.graphers as g
# import cofffee.grapher as go
# import cofffee.graph_props as m

# xls = pd.ExcelFile(
#     "/Users/brianmasse/Developer/Classes/CSC630/independent_work/cofffee Chain Visualization/PART I/data/cofffee Chain.xlsx"
# )
# data = pd.read_excel(xls, "cofffee Chain")

# width = 1700
# height = 900

# active_pallett = p.blue_lagoon

# handler = pg.handler( width, height, title="cofffee Shop Data")
# main = go.Grapher(handler, (1700, 1000))

# thick_line = m.line(
#         color=(255, 0, 0),
#         stroke=5
#     )

# area = g.area(main, data, "count()", "Marketing", "Product Type",
#     domain=m.domain(
#         pos=(100, 550),
#         size=( 1200, 300 )),
#     line=thick_line,
#     point=m.point(
#         color=(0, 0, 0),
#         shape=3
#     ),
#     title=m.text(
#         fontSize=25,
#         text="Quantity of Marketing Campaigns for Various Beverages",
#     ),
#     value_axis=ax.axis(
#         line=thick_line,
#         ticks=thick_line,
#         labels=m.text(
#             color=(255, 0, 0),
#             fontSize=10
#         ),
#     ),
#     pallett=active_pallett,
# )


# main.graphs = [ area ]
# main.handler.screen.fill( active_pallett.back_RGB )
# main.render()

# handler.start()
